@@ -1,7 +1,7 @@
 import shutil
 import tempfile
 from numpy import array, array_equal, add
-from thunder.utils.matrices import RowMatrix
+from thunder.rdds.matrices import RowMatrix
 from test_utils import PySparkTestCase
 
 
@@ -20,13 +20,13 @@ class TestElementWise(MatrixRDDTestCase):
     def test_elementwise_rdd(self):
         mat1 = RowMatrix(self.sc.parallelize([(1, array([1, 2, 3])), (2, array([4, 5, 6]))], 2))
         mat2 = RowMatrix(self.sc.parallelize([(1, array([7, 8, 9])), (2, array([10, 11, 12]))], 2))
-        result = mat1.elementwise(mat2, add).collect()
+        result = mat1.elementwise(mat2, add).rows().collect()
         truth = array([[8, 10, 12], [14, 16, 18]])
         assert array_equal(result, truth)
 
     def test_elementwise_array(self):
         mat = RowMatrix(self.sc.parallelize([(1, array([1, 2, 3]))]))
-        assert array_equal(mat.elementwise(2, add).collect()[0], array([3, 4, 5]))
+        assert array_equal(mat.elementwise(2, add).rows().collect()[0], array([3, 4, 5]))
 
 
 class TestTimes(MatrixRDDTestCase):
@@ -36,16 +36,16 @@ class TestTimes(MatrixRDDTestCase):
         mat2 = RowMatrix(self.sc.parallelize([(1, array([7, 8, 9])), (2, array([10, 11, 12]))], 2))
         truth = array([[47, 52, 57], [64, 71, 78], [81, 90, 99]])
         resultA = mat1.times(mat2)
-        resultB = mat1.times(mat2, "accum")
         assert array_equal(resultA, truth)
-        assert array_equal(resultB, truth)
 
     def test_times_array(self):
         mat1 = RowMatrix(self.sc.parallelize([(1, array([1, 2, 3])), (2, array([4, 5, 6]))]))
         mat2 = array([[7, 8], [9, 10], [11, 12]])
         truth = [array([58, 64]), array([139, 154])]
-        result = mat1.times(mat2).collect()
+        rdd = mat1.times(mat2)
+        result = rdd.rows().collect()
         assert array_equal(result, truth)
+        assert array_equal(rdd.index, range(0, 2))
 
 
 class TestOuter(MatrixRDDTestCase):
@@ -59,5 +59,3 @@ class TestOuter(MatrixRDDTestCase):
         assert array_equal(resultA, truth)
         assert array_equal(resultB1, truth)
         assert array_equal(resultB2, truth)
-
-# TODO: TestCenter, TestZScore
\ No newline at end of file
