@@ -59,17 +59,31 @@ def appendExtensionToPathSpec(datapath, ext=None):
     """
     if ext:
         if '*' in datapath:
-            if datapath[-1] == '*':
-                # path ends in wildcard but without postfix
-                # use ext as postfix
-                return datapath + ext
-            else:
-                # ext specified, but datapath apparently already has a postfix
-                # drop ext and use existing postfix
-                return datapath
+            # we already have a literal wildcard, which we take as a sign that the user knows
+            # what they're doing and don't want us overriding their path by appending extensions to it
+            return datapath
+        elif os.path.splitext(datapath)[1]:
+            # looks like we already have a literal extension specified at the end of datapath.
+            # go with that.
+            return datapath
         else:
             # no wildcard in path yet
-            return datapath+'*'+ext
+            # check whether we already end in `ext`, which suggests we've been passed a literal filename.
+            # prepend '.' to ext, as mild protection against the case where we have a directory 'bin' and
+            # are looking in it for files named '*.bin'.
+            if not ext.startswith('.'):
+                ext = '.'+ext
+            if not datapath.endswith(ext):
+                # we have an extension and we'd like to append it.
+                # we assume that datapath should be pointing to a directory at this point, but we might
+                # or might not have a directory separator at the end of it. add it if we don't.
+                if not datapath.endswith(os.path.sep):
+                    datapath += os.path.sep
+                # return a path with "/*."+`ext` added to it.
+                return datapath+'*'+ext
+            else:
+                # we are asking to append `ext`, but it looks like datapath already ends with '.'+`ext`
+                return datapath
     else:
         return datapath
 
@@ -242,27 +256,13 @@ class _BotoS3Client(object):
     def __init__(self):
         """Initialization; validates that AWS keys are available as environment variables.
 
-        Expects to find "AWS_ACCESS_KEY_ID" and "AWS_SECRET_ACCESS_KEY" in os.environ dict. Will throw
-        ValueError if either environment variable is missing.
+        Will let boto library look up credentials itself according to its own rules - e.g. first looking for
+        AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY, then going through several possible config files and finally
+        looking for a ~/.aws/credentials .ini-formatted file. See boto docs:
+        http://boto.readthedocs.org/en/latest/boto_config_tut.html
         """
         if not _have_boto:
             raise ValueError("The boto package does not appear to be available; boto is required for BotoS3Reader")
-        if (not 'AWS_ACCESS_KEY_ID' in os.environ) or (not 'AWS_SECRET_ACCESS_KEY' in os.environ):
-            raise ValueError("The environment variables 'AWS_ACCESS_KEY_ID' and 'AWS_SECRET_ACCESS_KEY' " +
-                             "must be set in order to read from s3")
-
-        # save keys in this object and serialize out to workers to prevent having to set env vars separately on all
-        # nodes in the cluster
-        self._access_key = os.environ['AWS_ACCESS_KEY_ID']
-        self._secret_key = os.environ['AWS_SECRET_ACCESS_KEY']
-
-    @property
-    def accessKey(self):
-        return self._access_key
-
-    @property
-    def secretKey(self):
-        return self._secret_key
 
 
 class BotoS3ParallelReader(_BotoS3Client):
@@ -276,7 +276,7 @@ class BotoS3ParallelReader(_BotoS3Client):
     def _listFiles(self, datapath, startidx=None, stopidx=None):
         parse = _BotoS3Client.parseS3Query(datapath)
 
-        conn = boto.connect_s3(aws_access_key_id=self.accessKey, aws_secret_access_key=self.secretKey)
+        conn = boto.connect_s3()
         bucket = conn.get_bucket(parse[0])
         keys = _BotoS3Client.retrieveKeys(bucket, parse[1], prefix=parse[2], postfix=parse[3])
         keynamelist = [key.name for key in keys]
@@ -297,16 +297,11 @@ class BotoS3ParallelReader(_BotoS3Client):
         if not keynamelist:
             raise FileNotFoundError("No S3 objects found for '%s'" % datapath)
 
-        access_key = self.accessKey
-        secret_key = self.secretKey
-
         def readSplitFromS3(kvIter):
-            conn = boto.connect_s3(aws_access_key_id=access_key, aws_secret_access_key=secret_key)
-            # bucket = conn.get_bucket(bucketname, validate=False)
+            conn = boto.connect_s3()
             bucket = conn.get_bucket(bucketname)
             for kv in kvIter:
                 idx, keyname = kv
-                #key = bucket.get_key(keyname, validate=False)
                 key = bucket.get_key(keyname)
                 buf = key.get_contents_as_string()
                 yield idx, buf
@@ -360,7 +355,7 @@ class BotoS3FileReader(_BotoS3Client):
     """
     def __getMatchingKeys(self, datapath, filename=None):
         parse = _BotoS3Client.parseS3Query(datapath)
-        conn = boto.connect_s3(aws_access_key_id=self.accessKey, aws_secret_access_key=self.secretKey)
+        conn = boto.connect_s3()
         bucketname = parse[0]
         keyname = parse[1]
         bucket = conn.get_bucket(bucketname)
@@ -445,11 +440,19 @@ class BotoS3ReadFileHandle(object):
         self._offset = 0
 
     def close(self):
-        self._key.close(fast=True)
+        try:
+            self._key.close(fast=True)
+        except TypeError:
+            # workaround for early versions of boto that don't have the 'fast' keyword
+            self._key.close()
         self._closed = True
 
     def read(self, size=-1):
         if self._offset or (size > -1):
+            # return empty string to indicate EOF if we are offset past the end of the file
+            # else boto will throw an error at us
+            if self._offset >= self._key.size:
+                return ""
             if size > -1:
                 sizestr = str(self._offset + size - 1)  # range header is inclusive
             else:
