@@ -2,16 +2,19 @@
 """
 from collections import namedtuple
 import json
-from numpy import array, dtype, frombuffer, arange, load, vstack, unravel_index
+from numpy import array, arange, dtype, frombuffer, load, ndarray, unravel_index, vstack
 from scipy.io import loadmat
 from cStringIO import StringIO
+import itertools
 import struct
 import urlparse
 import math
 
 from thunder.rdds.fileio.writers import getParallelWriterForPath
 from thunder.rdds.imageblocks import ImageBlocks
-from thunder.rdds.fileio.readers import getFileReaderForPath, FileNotFoundError, selectByStartAndStopIndices
+from thunder.rdds.keys import Dimensions
+from thunder.rdds.fileio.readers import getFileReaderForPath, FileNotFoundError, selectByStartAndStopIndices, \
+    appendExtensionToPathSpec
 from thunder.rdds.series import Series
 from thunder.utils.common import parseMemoryString
 
@@ -33,27 +36,60 @@ class SeriesLoader(object):
         self.sc = sparkcontext
         self.minPartitions = minPartitions
 
+    def fromArrays(self, arrays):
+        """Create a Series object from a sequence of numpy ndarrays resident in memory on the driver.
+
+        The arrays will be interpreted as though each represents a single time point - effectively the same
+        as if converting Images to a Series, with each array representing a volume image at a particular
+        point in time. Thus in the resulting Series, the value of the record with key (0,0,0) will be
+        array([arrays[0][0,0,0], arrays[1][0,0,0],... arrays[n][0,0,0]).
+
+        The dimensions of the resulting Series will be *opposite* that of the passed numpy array.
+        """
+        # if passed a single array, cast it to a sequence of length 1
+        if isinstance(arrays, ndarray):
+            arrays = [arrays]
+
+        # check that shapes of passed arrays are consistent
+        shape = arrays[0].shape
+        dtype = arrays[0].dtype
+        for ary in arrays:
+            if not ary.shape == shape:
+                raise ValueError("Inconsistent array shapes: first array had shape %s, but other array has shape %s" %
+                                 (str(shape), str(ary.shape)))
+            if not ary.dtype == dtype:
+                raise ValueError("Inconsistent array dtypes: first array had dtype %s, but other array has dtype %s" %
+                                 (str(dtype), str(ary.dtype)))
+
+        # get indices so that fastest index changes first
+        shapeiters = (xrange(n) for n in shape)
+        keys = [idx[::-1] for idx in itertools.product(*shapeiters)]
+
+        values = vstack([ary.ravel() for ary in arrays]).T
+
+        dims = Dimensions.fromTuple(shape[::-1])
+
+        return Series(self.sc.parallelize(zip(keys, values), self.minPartitions), dims=dims, dtype=dtype)
+
     @staticmethod
     def __normalizeDatafilePattern(datapath, ext):
-        if ext:
-            if not ext.startswith("."):
-                # protect (partly) against case where ext happens to *also* be the name
-                # of a directory. If your directory is named "something.bin", well, you
-                # get what you deserve, I guess.
-                ext = "." + ext
-            if not datapath.endswith(ext):
-                if datapath.endswith("*"):
-                    datapath += ext
-                elif datapath.endswith("/"):
-                    datapath += "*" + ext
-                else:
-                    datapath += "/*" + ext
+        datapath = appendExtensionToPathSpec(datapath, ext)
+        # we do need to prepend a scheme here, b/c otherwise the Hadoop based readers
+        # will adopt their default behavior and start looking on hdfs://.
 
         parseresult = urlparse.urlparse(datapath)
         if parseresult.scheme:
             # this appears to already be a fully-qualified URI
             return datapath
         else:
+            # this looks like a local path spec
+            # check whether we look like an absolute or a relative path
+            import os
+            dircomponent, filecomponent = os.path.split(datapath)
+            if not os.path.isabs(dircomponent):
+                # need to make relative local paths absolute; our file scheme parsing isn't all that it could be.
+                dircomponent = os.path.abspath(dircomponent)
+                datapath = os.path.join(dircomponent, filecomponent)
             return "file://" + datapath
 
     def fromText(self, datafile, nkeys=None, ext="txt"):
@@ -79,7 +115,7 @@ class SeriesLoader(object):
 
         lines = self.sc.textFile(datafile, self.minPartitions)
         data = lines.map(lambda x: parse(x, nkeys))
-
+        # TODO: allow a dtype to be specified in function args?
         return Series(data)
 
     BinaryLoadParameters = namedtuple('BinaryLoadParameters', 'nkeys nvalues keytype valuetype')
@@ -162,7 +198,7 @@ class SeriesLoader(object):
                          (tuple(int(x) for x in frombuffer(buffer(v, 0, keysize), dtype=keydtype)),
                           frombuffer(buffer(v, keysize), dtype=valdtype)))
 
-        return Series(data)
+        return Series(data, dtype=valdtype, index=arange(paramsObj.nvalues))
 
     def _getSeriesBlocksFromStack(self, datapath, dims, ext="stack", blockSize="150M", datatype='int16',
                                   startidx=None, stopidx=None):
@@ -177,6 +213,8 @@ class SeriesLoader(object):
             must be valid on all workers. Datafile may also refer to a single file, or to a range of files specified
             by a glob-style expression using a single wildcard character '*'.
 
+        dims: tuple of positive int
+            Dimensions of input image data, ordered with the fastest-changing dimension first.
 
         Returns
         ---------
@@ -195,7 +233,6 @@ class SeriesLoader(object):
             number of time points in returned series, determined from number of stack files found at datapath
 
         """
-
         datapath = self.__normalizeDatafilePattern(datapath, ext)
         blockSize = parseMemoryString(blockSize)
         totaldim = reduce(lambda x_, y_: x_*y_, dims)
@@ -215,7 +252,7 @@ class SeriesLoader(object):
             # different planes appear in distinct files
             blocksperplane = max(nblocks / dims[-1], 1)
 
-            pixperplane = reduce(lambda x_, y_: x_*y_, dims[:-1])
+            pixperplane = reduce(lambda x_, y_: x_*y_, dims[:-1])  # all but last dimension
 
             # get the greatest number of blocks in a plane (up to as many as requested) that still divide the plane
             # evenly. This will always be at least one.
@@ -280,7 +317,9 @@ class SeriesLoader(object):
             pass
 
         # get dimensions
-        dims = (firstifd.getImageWidth(), firstifd.getImageHeight(), len(tiffheaders.ifds))
+        npages = len(tiffheaders.ifds)
+        height = firstifd.getImageHeight()
+        width = firstifd.getImageWidth()
 
         # get datatype
         bitspersample = firstifd.getBitsPerSample()
@@ -299,14 +338,14 @@ class SeriesLoader(object):
                              % sampleformat)
         datatype = dtstr+str(bitspersample)
 
-        return dims, datatype
+        return height, width, npages, datatype
 
     def _getSeriesBlocksFromMultiTif(self, datapath, ext="tif", blockSize="150M",
                                      startidx=None, stopidx=None):
         import thunder.rdds.fileio.multitif as multitif
         import itertools
         from PIL import Image
-        from matplotlib.image import pil_to_array
+        from thunder.utils.common import pil_to_array
         import io
 
         datapath = self.__normalizeDatafilePattern(datapath, ext)
@@ -319,11 +358,13 @@ class SeriesLoader(object):
         filenames = selectByStartAndStopIndices(filenames, startidx, stopidx)
         ntimepoints = len(filenames)
 
-        dims, datatype = SeriesLoader.__readMetadataFromFirstPageOfMultiTif(reader, filenames[0])
+        minimize_reads = datapath.lower().startswith("s3")
+
+        height, width, npages, datatype = SeriesLoader.__readMetadataFromFirstPageOfMultiTif(reader, filenames[0])
         pixelbytesize = dtype(datatype).itemsize
 
         # intialize at one block per plane
-        bytesperplane = dims[0] * dims[1] * pixelbytesize * ntimepoints
+        bytesperplane = height * width * pixelbytesize * ntimepoints
         bytesperblock = bytesperplane
         blocksperplane = 1
         # keep dividing while cutting our size in half still leaves us bigger than the requested size
@@ -332,10 +373,12 @@ class SeriesLoader(object):
             bytesperblock /= 2
             blocksperplane *= 2
 
-        blocklen = max((dims[0] * dims[1]) / blocksperplane, 1)  # integer division
+        blocklenPixels = max((height * width) / blocksperplane, 1)  # integer division
+        while blocksperplane * blocklenPixels < height * width:  # make sure we're reading the plane fully
+            blocksperplane += 1
 
         # keys will be planeidx, blockidx:
-        keys = list(itertools.product(xrange(dims[2]), xrange(blocksperplane)))
+        keys = list(itertools.product(xrange(npages), xrange(blocksperplane)))
 
         def readblockfromtif(pidxbidx_):
             planeidx, blockidx = pidxbidx_
@@ -347,16 +390,31 @@ class SeriesLoader(object):
                 reader_ = getFileReaderForPath(fname)()
                 fp = reader_.open(fname)
                 try:
-                    tiffparser_ = multitif.TiffParser(fp, debug=False)
-                    tiffilebuffer = multitif.packSinglePage(tiffparser_, page_idx=planeidx)
-                    pilimg = Image.open(io.BytesIO(tiffilebuffer))
-                    ary = pil_to_array(pilimg)
-                    del tiffilebuffer, tiffparser_, pilimg
+                    if minimize_reads:
+                        # use multitif module to generate a fake, in-memory one-page tif file
+                        # the advantage of this is that it cuts way down on the many small reads
+                        # that PIL/pillow will make otherwise, which would be a problem for s3
+                        tiffparser_ = multitif.TiffParser(fp, debug=False)
+                        tiffilebuffer = multitif.packSinglePage(tiffparser_, page_idx=planeidx)
+                        bytebuf = io.BytesIO(tiffilebuffer)
+                        try:
+                            pilimg = Image.open(bytebuf)
+                            ary = pil_to_array(pilimg).T
+                        finally:
+                            bytebuf.close()
+                        del tiffilebuffer, tiffparser_, pilimg, bytebuf
+                    else:
+                        # read tif using PIL directly
+                        pilimg = Image.open(fp)
+                        pilimg.seek(planeidx)
+                        ary = pil_to_array(pilimg).T
+                        del pilimg
+
                     if not planeshape:
                         planeshape = ary.shape[:]
-                        blockstart = blockidx * blocklen
-                        blockend = min(blockstart+blocklen, planeshape[0]*planeshape[1])
-                    blocks.append(ary.flatten(order='C')[blockstart:blockend])
+                        blockstart = blockidx * blocklenPixels
+                        blockend = min(blockstart+blocklenPixels, planeshape[0]*planeshape[1])
+                    blocks.append(ary.ravel(order='C')[blockstart:blockend])
                     del ary
                 finally:
                     fp.close()
@@ -374,6 +432,13 @@ class SeriesLoader(object):
 
         # map over blocks
         rdd = self.sc.parallelize(keys, len(keys)).flatMap(readblockfromtif)
+        # hack for returned dimensions:
+        # if npages == 1:
+        #     dims = (width, height, npages)
+        # else:
+        #     dims = (npages, width, height)
+        dims = (npages, width, height)
+
         metadata = (dims, ntimepoints, datatype)
         return rdd, metadata
 
@@ -388,7 +453,7 @@ class SeriesLoader(object):
             including scheme. A datapath argument may include a single '*' wildcard character in the filename.
 
         dims: tuple of positive int
-            Dimensions of input image data, similar to a numpy 'shape' parameter.
+            Dimensions of input image data, ordered with the fastest-changing dimension first.
 
         ext: string, optional, default "stack"
             Extension required on data files to be loaded.
@@ -406,8 +471,8 @@ class SeriesLoader(object):
         seriesblocks, npointsinseries = self._getSeriesBlocksFromStack(datapath, dims, ext=ext, blockSize=blockSize,
                                                                        datatype=datatype, startidx=startidx,
                                                                        stopidx=stopidx)
-        # TODO: initialize index here using npointsinseries?
-        return Series(seriesblocks, dims=dims)
+
+        return Series(seriesblocks, dims=dims, dtype=datatype, index=arange(npointsinseries))
 
     def fromMultipageTif(self, datapath, ext="tif", blockSize="150M",
                          startidx=None, stopidx=None):
@@ -433,7 +498,7 @@ class SeriesLoader(object):
         seriesblocks, metadata = self._getSeriesBlocksFromMultiTif(datapath, ext=ext, blockSize=blockSize,
                                                                    startidx=startidx, stopidx=stopidx)
         dims, npointsinseries, datatype = metadata
-        return Series(seriesblocks, dims=dims)
+        return Series(seriesblocks, dims=Dimensions.fromTuple(dims[::-1]), dtype=datatype, index=arange(npointsinseries))
 
     @staticmethod
     def __saveSeriesRdd(seriesblocks, outputdirname, dims, npointsinseries, datatype, overwrite=False):
@@ -472,7 +537,7 @@ class SeriesLoader(object):
             on the local file system or a URI-like format, as in datapath.
 
         dims: tuple of positive int
-            Dimensions of input image data, similar to a numpy 'shape' parameter.
+            Dimensions of input image data, ordered with the fastest-changing dimension first.
 
         ext: string, optional, default "stack"
             Extension required on data files to be loaded.
@@ -544,7 +609,7 @@ class SeriesLoader(object):
         else:
             keys = arange(0, data.shape[0])
 
-        rdd = Series(self.sc.parallelize(zip(keys, data), self.minPartitions))
+        rdd = Series(self.sc.parallelize(zip(keys, data), self.minPartitions), dtype=data.dtype)
 
         return rdd
 
@@ -561,7 +626,7 @@ class SeriesLoader(object):
         else:
             keys = arange(0, data.shape[0])
 
-        rdd = Series(self.sc.parallelize(zip(keys, data), self.minPartitions))
+        rdd = Series(self.sc.parallelize(zip(keys, data), self.minPartitions), dtype=data.dtype)
 
         return rdd
 
