@@ -1,6 +1,7 @@
 from numpy import ndarray, arange, amax, amin, size, squeeze, dtype
 
 from thunder.rdds.data import Data
+from thunder.rdds.keys import Dimensions
 from thunder.utils.common import parseMemoryString
 
 
@@ -12,19 +13,22 @@ class Images(Data):
     is an identifier and the value is a two or three-dimensional array.
     """
 
-    _metadata = ['_dims', '_nimages', '_dtype']
+    _metadata = Data._metadata + ['_dims', '_nimages']
 
     def __init__(self, rdd, dims=None, nimages=None, dtype=None):
-        super(Images, self).__init__(rdd)
-        # todo: add parameter checking here?
+        super(Images, self).__init__(rdd, dtype=dtype)
+        if dims and not isinstance(dims, Dimensions):
+            try:
+                dims = Dimensions.fromTuple(dims)
+            except:
+                raise TypeError("Images dims parameter must be castable to Dimensions object, got: %s" % str(dims))
         self._dims = dims
         self._nimages = nimages
-        self._dtype = str(dtype) if dtype else None
 
     @property
     def dims(self):
         if self._dims is None:
-            self._populateParamsFromFirstRecord()
+            self.populateParamsFromFirstRecord()
         return self._dims
 
     @property
@@ -35,18 +39,17 @@ class Images(Data):
 
     @property
     def dtype(self):
-        if not self._dtype:
-            self._populateParamsFromFirstRecord()
-        return self._dtype
+        # override just calls superclass; here for explicitness
+        return super(Images, self).dtype
 
     @property
     def _constructor(self):
         return Images
 
-    def _populateParamsFromFirstRecord(self):
-        record = self.rdd.first()
-        self._dims = record[1].shape
-        self._dtype = str(record[1].dtype)
+    def populateParamsFromFirstRecord(self):
+        record = super(Images, self).populateParamsFromFirstRecord()
+        self._dims = Dimensions.fromTuple(record[1].shape)
+        return record
 
     def _resetCounts(self):
         self._nimages = None
@@ -101,11 +104,11 @@ class Images(Data):
             of an xy plane, divided in the middle of the y dimension
 
         """
-
+        # splitsPerDim is expected to be in the dimensions ordering convention
         import itertools
         from thunder.rdds.imageblocks import ImageBlocks, ImageBlockValue
 
-        dims = self.dims
+        dims = self.dims.count[:]  # currently in Dimensions-convention
         ndim = len(dims)
         totnumimages = self.nimages
 
@@ -133,6 +136,9 @@ class Images(Data):
                 st = en
             slices.append(dimslices)
 
+        # reverse slices to be in numpy shape ordering convention:
+        # slices = slices[::-1]
+
         def _groupBySlices(imagearyval, slices_, tp_, numtp_):
             ret_vals = []
             sliceproduct = itertools.product(*slices_)
@@ -321,7 +327,7 @@ class Images(Data):
             return binlabel+'.bin', binvals
 
         binseriesrdd.map(appendBin).foreach(writer.writerFcn)
-        writeSeriesConfig(outputdirname, len(self.dims), self.nimages, dims=self.dims,
+        writeSeriesConfig(outputdirname, len(self.dims), self.nimages, dims=self.dims.count,
                           keytype='int16', valuetype=self.dtype, overwrite=overwrite)
 
     def exportAsPngs(self, outputdirname, fileprefix="export", overwrite=False,
@@ -460,8 +466,8 @@ class Images(Data):
             zrange = arange(bottom+1, top)
         newdims = [self.dims[0], self.dims[1], size(zrange)]
 
-        return self._constructor(
-            self.rdd.mapValues(lambda v: squeeze(v[:, :, zrange])), dims=newdims).__finalize__(self)
+        return self._constructor(self.rdd.mapValues(lambda v: squeeze(v[:, :, zrange])),
+                                 dims=newdims).__finalize__(self)
 
     def subtract(self, val):
         """
