@@ -1,7 +1,8 @@
 from numpy import ndarray, array, sum, mean, std, size, arange, \
-    polyfit, polyval, percentile, float16, asarray, maximum, zeros, corrcoef, where
+    polyfit, polyval, percentile, asarray, maximum, zeros, corrcoef, where
 
 from thunder.rdds.data import Data
+from thunder.rdds.keys import Dimensions
 from thunder.utils.common import checkparams, loadmatvar
 
 
@@ -34,34 +35,56 @@ class Series(Data):
     SpatialSeries : a Series where the keys represent spatial coordinates
     """
 
-    _metadata = ['_index', '_dims']
+    _metadata = Data._metadata + ['_index', '_dims']
 
-    def __init__(self, rdd, index=None, dims=None):
-        super(Series, self).__init__(rdd)
+    def __init__(self, rdd, index=None, dims=None, dtype=None):
+        super(Series, self).__init__(rdd, dtype=dtype)
         self._index = index
-        if isinstance(dims, (tuple, list)):
-            from thunder.rdds.keys import Dimensions
-            self._dims = Dimensions.fromNumpyDimsTuple(dims)
-        else:
-            self._dims = dims
+        if dims and not isinstance(dims, Dimensions):
+            try:
+                dims = Dimensions.fromTuple(dims)
+            except:
+                raise TypeError("Series dims parameter must be castable to Dimensions object, got: %s" % str(dims))
+        self._dims = dims
 
     @property
     def index(self):
         if self._index is None:
-            record = self.rdd.first()
-            self._index = arange(0, len(record[1]))
+            self.populateParamsFromFirstRecord()
         return self._index
 
     @property
     def dims(self):
         from thunder.rdds.keys import Dimensions
         if self._dims is None:
-            entry = self.rdd.first()[0]
+            entry = self.populateParamsFromFirstRecord()[0]
             n = size(entry)
             d = self.rdd.keys().mapPartitions(lambda i: [Dimensions(i, n)]).reduce(lambda x, y: x.mergedims(y))
             self._dims = d
         return self._dims
 
+    @property
+    def dtype(self):
+        # override just calls superclass; here for explicitness
+        return super(Series, self).dtype
+
+    def populateParamsFromFirstRecord(self):
+        """Calls first() on the underlying rdd, using the returned record to determine appropriate attribute settings
+        for this object (for instance, setting self.dtype to match the dtype of the underlying rdd records).
+
+        Returns the result of calling self.rdd.first().
+        """
+        record = super(Series, self).populateParamsFromFirstRecord()
+        if self._index is None:
+            val = record[1]
+            try:
+                l = len(val)
+            except TypeError:
+                # TypeError thrown after calling len() on object with no __len__ method
+                l = 1
+            self._index = arange(0, l)
+        return record
+
     @property
     def _constructor(self):
         return Series
@@ -399,7 +422,8 @@ class Series(Data):
         """
         from thunder.rdds.keys import _subtoind_converter
 
-        converter = _subtoind_converter(self.dims.max, order=order, onebased=onebased)
+        # converter = _subtoind_converter(self.dims.max, order=order, onebased=onebased)
+        converter = _subtoind_converter(self.dims.count, order=order, onebased=onebased)
         rdd = self.rdd.map(lambda (k, v): (converter(k), v))
         return self._constructor(rdd, index=self._index).__finalize__(self)
 
@@ -427,7 +451,7 @@ class Series(Data):
         rdd = self.rdd.map(lambda (k, v): (converter(k), v))
         return self._constructor(rdd, index=self._index).__finalize__(self)
 
-    def pack(self, selection=None, sorting=False):
+    def pack(self, selection=None, sorting=False, transpose=False, dtype=None, casting='safe'):
         """
         Pack a Series into a local array (e.g. for saving)
 
@@ -446,12 +470,25 @@ class Series(Data):
             Whether to sort the local array based on the keys. In most cases the returned array will
             already be ordered correctly, and so an explicit sorting=True is typically not necessary.
 
+        transpose : boolean, optional, default False
+            Transpose the spatial dimensions of the returned array.
+
+        dtype: numpy dtype, dtype specifier, or string 'smallfloat'. optional, default None.
+            If present, will cast the values to the requested dtype before collecting on the driver. See Data.astype()
+            and numpy's astype() function for details.
+
+        casting: casting: 'no'|'equiv'|'safe'|'same_kind'|'unsafe', optional, default 'safe'
+            Casting method to pass on to numpy's astype() method if dtype is given; see numpy documentation for details.
+
         Returns
         -------
         result: numpy array
             An array with dimensionality inferred from the RDD keys. Data in an individual Series
             value will be placed into this returned array by interpreting the Series keys as indicies
-            into the returned array.
+            into the returned array. The shape of the returned array will be (num time points x spatial shape).
+            For instance, a series derived from 4 2d images, each 64 x 128, will have dims.count==(64, 128)
+            and will pack into an array with shape (4, 64, 128). If transpose is true, the spatial dimensions
+            will be reversed, so that in this example the shape of the returned array will be (4, 128, 64).
         """
 
         if selection:
@@ -459,7 +496,10 @@ class Series(Data):
         else:
             out = self
 
-        result = out.rdd.map(lambda (_, v): float16(v)).collect()
+        if not (dtype is None):
+            out = out.astype(dtype, casting)
+
+        result = out.rdd.map(lambda (_, v): v).collect()
         nout = size(result[0])
 
         if sorting is True:
@@ -470,11 +510,14 @@ class Series(Data):
         # where b is the number of outputs per record
         out = asarray(result).reshape(((nout,) + self.dims.count)[::-1]).T
 
-        # flip xy for spatial data
-        if size(self.dims.count) == 3:  # (b, x, y, z) -> (b, y, x, z)
-            out = out.transpose([0, 2, 1, 3])
-        if size(self.dims.count) == 2:  # (b, x, y) -> (b, y, x)
-            out = out.transpose([0, 2, 1])
+        if transpose:
+            # swap arrays so that in-memory representation matches that
+            # of original input. default is to return array whose shape matches
+            # that of the series dims object.
+            if size(self.dims.count) == 3:
+                out = out.transpose([0, 3, 2, 1])
+            if size(self.dims.count) == 2:  # (b, x, y) -> (b, y, x)
+                out = out.transpose([0, 2, 1])
 
         return out.squeeze()
 
@@ -499,18 +542,19 @@ class Series(Data):
             A local numpy array with the subset of points
         """
         from numpy.linalg import norm
+        from numpy.random import randint
 
-        stat_dict = {'std': std,
-                     'norm': norm}
+        stat_dict = {'std': std, 'norm': norm}
+        seed = randint(0, 2 ** 32 - 1)
 
         if thresh is not None:
             func = stat_dict[stat]
-            result = array(self.rdd.values().filter(lambda x: func(x) > thresh).takeSample(False, nsamples))
+            result = array(self.rdd.values().filter(lambda x: func(x) > thresh).takeSample(False, nsamples, seed=seed))
         else:
-            result = array(self.rdd.values().takeSample(False, nsamples))
+            result = array(self.rdd.values().takeSample(False, nsamples, seed=seed))
 
         if size(result) == 0:
-            raise Exception('No records found, try a different threshold?')
+            raise Exception('No records found, maybe threshold of %g is too high, try changing it?' % thresh)
 
         return result
 
@@ -546,7 +590,7 @@ class Series(Data):
         """
 
         if isinstance(inds, str):
-            inds = loadmatvar(inds, var)
+            inds = loadmatvar(inds, var)[0]
         else:
             inds = asarray(inds)
 
@@ -562,9 +606,9 @@ class Series(Data):
 
         for idx, indlist in enumerate(inds):
             if len(indlist) > 0:
-                inds_set = set(indlist)
+                inds_set = set(asarray(indlist).flat)
                 inds_bc = self.rdd.context.broadcast(inds_set)
-                values[idx, :] = data.filterOnKeys(lambda k: k in inds_bc.value).values().sum() / len(indlist)
+                values[idx, :] = data.filterOnKeys(lambda k: k in inds_bc.value).values().mean()
                 keys[idx, :] = mean(map(lambda k: converter(k), indlist), axis=0)
 
         return keys, values
