@@ -155,7 +155,7 @@ class Colorize(object):
             if img.ndim == 4:
                 out = zeros((img_dims[1], img_dims[2], img_dims[3], 3))
             if img.ndim == 3:
-                out = zeros((img_dims[1], img_dims[2]))
+                out = zeros((img_dims[1], img_dims[2], 3))
             for ix, clr in enumerate(self.colors):
                 cmap = LinearSegmentedColormap.from_list('blend', [[0, 0, 0], clr])
                 tmp = cmap(self.scale * base[ix]/amax(base[ix]))
@@ -243,7 +243,8 @@ class Colorize(object):
                     raise Exception('Must have 2 values per pixel for %s conversion' % self.totype)
             if self.totype in ['indexed']:
                 if dims[0] != len(self.colors):
-                    raise Exception('Must have %g values per pixel for %s conversion with given list' % self.totype)
+                    raise Exception('Must have %g values per pixel for %s conversion with given list'
+                                    % (len(self.colors), self.totype))
 
         elif isinstance(self.totype, ListedColormap) or isinstance(self.totype, str):
             if len(dims) not in [2, 3]:
@@ -296,7 +297,7 @@ class Colorize(object):
         optfun = lambda x: 1 - corrcoef(distmat, squareform(pdist(x.reshape(nclrs, 3), 'cosine')).flatten())[0, 1]
         init = random.rand(nclrs*3)
         bounds = [(0, 1) for _ in range(0, nclrs * 3)]
-        res = minimize(optfun, init, bounds=bounds)
+        res = minimize(optfun, init, bounds=bounds, method='L-BFGS-B')
         newclrs = res.x.reshape(nclrs, 3).tolist()
 
         from matplotlib.colors import ListedColormap
